----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		intIO - behave_intio 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		internal (onboard) IO controller and wishbone interface
--							includes 8 switches, 4 buttons, 8 LEDs, 4 seven-segment-displays
--
-- Dependencies:		softcore_pkg	 - softcore_pkg.vhd
--
-- Revision:			0.02
-- Revision 0.01 - File created
-- Revision 0.02 - Added functionality to control above mentioned IOs via the
--						 wishbone bus
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- Including dependencies
library work;
use work.softcore_pkg.all;



entity intIO is
    port (clk				: in  std_logic;
			 reset			: in  std_logic;
			 
			 interrupt		: out std_logic;
			 
			 slave_in		: in  wb_master_out;
			 slave_out		: out wb_master_in;
			 
			 -- IO signals:
			 -- 8 onboard LEDs
			 LEDs				: out std_logic_vector (7 downto 0);
			 -- 8 onboard switches
			 switches		: in std_logic_vector (7 downto 0);
			 -- 4 onboard 7-segment display
			 sseg_on			: out std_logic_vector (3 downto 0); -- activation signal for each display
			 sseg				: out std_logic_vector (7 downto 0); -- 8 bit value for the segments
			 -- 4 onboard buttons
			 buttons			: in std_logic_vector (3 downto 0)
			 );
end intIO;

architecture behave_intio of intIO is

	signal LED							: std_logic_vector (7 downto 0);
	signal S_SEG						: std_logic_vector (31 downto 0);
	signal S_SEG_part					: std_logic_vector (7 downto 0);
	signal switch_segment_led_on	: std_logic_vector (3 downto 0);
	
begin
	-- Interrupt not used, tie it to 0
	interrupt <= '0';

	-- The slave acknowledge is set by the cyc signal
	slave_out.ack <= slave_in.cyc;
	
	
	
	-- The slave data to be sent via the wishbone bus to the ZPU are set with the values of a
	-- specific IO when its address is selected
	-- for this currently the last to bits are used to distinguish between the IOs
	-- 00 - Buttons
	-- 10 - LEDs (just as feedback, because the LEDs are output signals)
	-- 01 - Switches
	-- 11 - 4 seven-segment-displays (just as feedback, because the seven-segment-displays are output signals)
	slave_out.data <= (others => '0') when (reset = '1') else
							"000000000000000000000000" & LED when (slave_in.cyc = '1' and slave_in.addr (1 downto 0) = "10") else
							"000000000000000000000000" & switches when (slave_in.cyc = '1' and slave_in.addr (1 downto 0) = "01") else
							S_SEG when (slave_in.cyc = '1' and slave_in.addr (1 downto 0) = "11") else
							"0000000000000000000000000000" & buttons when (slave_in.cyc = '1' and slave_in.addr (1 downto 0) = "00") else
							(others => '0');


	-- When the IO address is selected, set the IO signals by using the wishbone input data from the ZPU
	LED   <= (others => '0') when (reset = '1') else
				slave_in.data (7 downto 0) when (slave_in.cyc = '1' and slave_in.we = '1' and slave_in.addr (1 downto 0) = "10");
	S_SEG <= (others => '0') when (reset = '1') else
				slave_in.data (31 downto 0) when (slave_in.cyc = '1' and slave_in.we = '1' and slave_in.addr (1 downto 0) = "11");
	
	
	LEDs	<= LED;
	
	
	sseg <= not S_SEG_part; -- the signal is inverted for easier use because 1 normally means off and 0 means on
	sseg_on <= switch_segment_led_on;
	
	
	-- Segment control
	-- only one of the 4 seven-segment-displays can set at one time so a state machine is used
	-- to set all 4. So a 32 bit value can be used to set all of them
	segment_led_ctrl:
	process (clk, reset)
		type segment_state_type is
		(
			segment0,
			segment1,
			segment2,
			segment3
		);
		variable segment_state : segment_state_type;
		variable r_reg : unsigned (31 downto 0);
		
	begin
		if (reset = '1') then
			segment_state 			 := segment0;
			S_SEG_part 	  			 <= (others => '0');
			switch_segment_led_on <= (others => '0');
			r_reg			  			 := (others => '0');
			
		elsif (clk'event and clk = '1') then
			if (r_reg >= 50000) then
				r_reg := (others => '0');

				-- State machine to toggle between the 4 displays
				case segment_state is
					when segment0 =>
						switch_segment_led_on <= "0111";
						S_SEG_part <= S_SEG (31 downto 24);
						segment_state := segment1;
					
					when segment1 =>						
						switch_segment_led_on <= "1011";
						S_SEG_part <= S_SEG (23 downto 16);
						segment_state := segment2;
					
					when segment2 =>
						switch_segment_led_on <= "1101";
						S_SEG_part <= S_SEG (15 downto 8);
						segment_state := segment3;
					
					when segment3 =>
						switch_segment_led_on <= "1110";
						S_SEG_part <= S_SEG (7 downto 0);
						segment_state := segment0;
					
					when others =>
						switch_segment_led_on <= "0111";
						S_SEG_part <= S_SEG (31 downto 24);
						segment_state := segment1;
				end case;
			else
				r_reg := r_reg + 1;
			end if;
		end if;
	end process segment_led_ctrl;
	
end behave_intio;

