----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		interrupt - behave_interrupt 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		Interrupt controller and wishbone interface
--
-- Dependencies:		softcore_pkg	 - softcore_pkg.vhd
--
-- Revision:			0.01
-- Revision 0.01 - File received from Matthias Meier
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


-- Including dependencies
library work;
use work.softcore_pkg.all;



entity interrupt is
	generic (irqLines	: integer := 0);
	port (clk			: in  std_logic;
	
		 reset			: in  std_logic;
		 
		 irq_core		: out std_logic;
		 irq				: in  std_logic_vector (irqLines - 1 downto 0);
		 
		 slave_in		: in  wb_master_out;
		 slave_out		: out wb_master_in
		 );
end entity interrupt;



architecture behave_interrupt of interrupt is

signal globalEnable	  : std_logic;
signal interruptEnable : std_logic_vector (irqLines - 1 downto 0);

begin

	-- Set interrupt to the ZPU
	irq_core <= '1' when (((irq and interruptEnable) /= std_logic_vector (to_unsigned (0, irqLines))) and (globalEnable = '1')) else '0';

	-- The slave acknowledge is set by the cyc signal
	slave_out.ack <= slave_in.cyc;
	
	-- The slave data to be sent via the wishbone bus to the ZPU are set with the following options
	slave_out.data <=
						-- Reading the IRQs
						std_logic_vector (to_unsigned (0, (31 - irqLines + 1))) & (irq and interruptEnable) when (slave_in.cyc = '1' and slave_in.addr (3 downto 2) = "01") else
						-- Reading the interrupt
						std_logic_vector (to_unsigned (0, (31 - irqLines + 1))) & interruptEnable when (slave_in.cyc = '1' and slave_in.addr (3 downto 2) = "10") else
						-- Reading the global Interrupt-Flag
						"0000000000000000000000000000000" & globalEnable when (slave_in.cyc = '1' and slave_in.addr (3 downto 2) = "11") else										
						(others => '0');


	interrupt_process:
	process (clk, reset)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				globalEnable <= '0';
				interruptEnable <= (others => '0');
			else
				if (slave_in.cyc = '1' and slave_in.we = '1') then
					-- Set / reset interrupt
					if (slave_in.addr (3 downto 2) = "10") then
						interruptEnable <= slave_in.data(irqLines - 1 downto 0);

					-- Set / reset the global Interrupt-Flag
					elsif (slave_in.addr (3 downto 2) = "11") then
						globalEnable <= slave_in.data(0);
					end if;
				end if;
			end if;
		end if;
	end process;
	
end architecture behave_interrupt;
