----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		MemoryController - behave_mem_controller 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		SRAM memory controller
--
-- Dependencies:		no
--
-- Revision:			0.01
-- Revision 0.01 - File received from Dennis Trebbels, trebbelsd@gmx.de
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;




entity MemoryController is
    Port (clk			: in  STD_LOGIC;
			
			reset			: in  STD_LOGIC;
         
			mem			: in  STD_LOGIC; -- chip enable
			rw				: in  STD_LOGIC; -- read: 1, write: 0
			addr			: in  STD_LOGIC_VECTOR (17 downto 0); -- address where to read or write data
			data_f2s		: in  STD_LOGIC_VECTOR (15 downto 0); -- data to put to the SRAM
			ready			: out  STD_LOGIC; -- ready: 1, busy: 0
			data_s2f_r	: out  STD_LOGIC_VECTOR (15 downto 0); -- data from SRAM
			data_s2f_ur	: out  STD_LOGIC_VECTOR (15 downto 0); -- not used in this design
						
			-- onboard SRAM1 (ISSI 256Kx16 SRAM, 10ns)
			-- Ports to the SRAM
			ad				: out std_logic_vector (17 downto 0); -- address bus (18 Bit)
			we_n			: out std_logic; -- write enable
			oe_n			: out std_logic; -- output enable
			dio_a			: inout std_logic_vector (15 downto 0); -- data bus (16 Bit)
			ce_a_n		: out std_logic; -- chip enable
			ub_a_n		: out std_logic; -- upper byte enable
			lb_a_n		: out std_logic -- lower byte enable
			);
end MemoryController;

architecture behave_mem_controller of MemoryController is

type state_type is (idle, rd1, rd2, wr1, wr2);

	signal state_reg, state_next			: state_type;
	signal data_f2s_reg, data_f2s_next	: std_logic_vector (15 downto 0);
	signal data_s2f_reg, data_s2f_next	: std_logic_vector (15 downto 0);
	signal addr_reg, addr_next				: std_logic_vector (17 downto 0);
	signal we_buf, oe_buf, tri_buf		: std_logic;
	signal we_reg, oe_reg, tri_reg		: std_logic;

begin

	-- State & data registers
	process (clk, reset)
	begin
		-- After a reset state machine should start in the idle state
		-- address is then initializied with zero
		if (reset='1') then
			state_reg 		<= idle;
			addr_reg 		<= (others => '0');
			data_f2s_reg 	<= (others => '0');
			data_s2f_reg 	<= (others => '0');
			tri_reg 			<= '1';
			we_reg 			<= '1';
			oe_reg 			<= '1';

		elsif (clk'event and clk='1') then
			state_reg 		<= state_next;
			addr_reg 		<= addr_next;
			data_f2s_reg 	<= data_f2s_next;
			data_s2f_reg 	<= data_s2f_next;
			tri_reg 			<= tri_buf;
			we_reg 			<= we_buf;
			oe_reg 			<= oe_buf;
		end if;	
	end process;

	-- next-state logic
	process (state_reg, mem, rw, dio_a, addr, data_f2s, data_f2s_reg, data_s2f_reg, addr_reg)
	begin
		addr_next 		<= addr_reg;
		data_f2s_next 	<= data_f2s_reg;
		data_s2f_next 	<= data_s2f_reg;
		ready 			<= '0';
		
		case state_reg is
			when idle =>
				if (mem = '0') then
					state_next <= idle;
				else
					addr_next <= addr;
					
					if (rw = '0') then --write
						state_next 		<= wr1;
						data_f2s_next 	<= data_f2s;
					else -- read
						state_next 		<= rd1;
					end if;
				end if;
				ready <= '1';
			when wr1 =>
				state_next <= wr2;
			when wr2 =>
				state_next <= idle;
			when rd1 =>
				state_next <= rd2;
			when rd2=>
				data_s2f_next 	<= dio_a;
				state_next 		<= idle;
		end case;
	end process;

	-- next-state logic
	process (state_next)
	begin
		tri_buf	<= '1'; -- signals are active low
		we_buf	<= '1';
		oe_buf	<= '1';
		
		case state_next is
			when idle =>
			when wr1 =>
				tri_buf	<= '0';
				we_buf	<= '0';
			when wr2 =>
				tri_buf	<= '0';
			when rd1 =>
				oe_buf	<= '0';
			when rd2=>
				oe_buf	<= '0';
		end case;
	end process;

	-- to main system
	data_s2f_r	<= data_s2f_reg;
	data_s2f_ur	<= dio_a;

	-- to sram
	we_n	<= we_reg;
	oe_n	<= oe_reg;
	ad		<= addr_reg;

	-- i/o for SRAM chip a
	ce_a_n	<='0';
	ub_a_n	<='0';
	lb_a_n	<='0';
	dio_a		<= data_f2s_reg when tri_reg = '0' else (others => 'Z');

end behave_mem_controller;
