----------------------------------------------------------------------------------
-- Company:					DHBW Mannheim, Germany
-- Engineer:				Johannes Steudle
-- 
-- Create Date:			11:00:00 02/18/2011 
-- Design Name:			ZPU
-- Module Name:			softcore_top - behave_softcore_top 
-- Project Name:			Softcore Implementation
-- Target Devices:		Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:			Xilinx ISE 10.1 WebPack
--
-- Project description:	This project contains an opensource softcore CPU (the ZPU
--								created by yvind Harboe). This is a small and very simple
--								stack based	32 bit CPU with an 8 bit opcode. The ZPU uses
--								a dualport RAM (internal BlockRAM) as program memory and
--								stack.
--								In this project a wishbone bus is connected to the ZPU to
--								communicate with peripherals.
--
-- Description:			Top level source for softcore ZPU and its peripheral devices
--								contains the wishbone system to connect the devices to the
--								ZPU. There is also an interrupt controller implemented.
--
-- Dependencies:			softcore_config - softcore_config.vhd
--								softcore_pkg	 - softcore_pkg.vhd
--
-- Revision:				0.02 
-- Revision 0.01 - File received from Matthias Meier
-- Revision 0.02 - Wishbone Communication added some new devices, now it contains:
--							Master: wb_core; Slaves: interrupt, uart, ps2, intIO,
--																wb_sram, wb_vga
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;


-- Including dependencies
library work;
use work.softcore_config.all;
use work.softcore_pkg.all;



-- Top level entity softcore_top
entity softcore_top is
	port (clk				: in  std_logic;
			 -- reset			: in  std_logic; -- see ioButtons(3)
			 
			 -- UART interface
			 uart_rx1		: in  std_logic; -- RX
			 uart_tx1		: out std_logic; -- TX
			 
			 -- PS/2 interface
			 ps2_clk			: inout std_logic; -- Clock
			 ps2_data		: inout std_logic; -- Data
			  
			 -- 8 onboard LEDs
			 ioLEDs			: out std_logic_vector (7 downto 0);
			 
			 -- 8 onboard switches
			 ioSwitches		: in std_logic_vector (7 downto 0);
			  
			 -- 4 onboard 7-segment display
			 sseg_on			: out std_logic_vector (3 downto 0); -- activation signal for each display
			 sseg				: out std_logic_vector (7 downto 0); -- 8 bit value for the segments
			   
			 -- 4 onboard buttons
			 ioButtons		: in std_logic_vector (3 downto 0);
			 
			 -- onboard SRAM1 (ISSI 256Kx16 SRAM, 10ns)
			 sram_adr		: out std_logic_vector (17 downto 0); -- address bus (18 Bit)
			 sram_we_n		: out std_logic; -- write enable
			 sram_oe_n		: out std_logic; -- output enable
			 sram_dio_a		: inout std_logic_vector (15 downto 0); -- data bus (16 Bit)
			 sram_ce_a_n	: out std_logic; -- chip enable
			 sram_ub_a_n	: out std_logic; -- upper byte enable
			 sram_lb_a_n	: out std_logic; -- lower byte enable
				 
			 -- VGA interface
			 ioVGA_hsync	: out std_logic; -- horizontal synchronisation
			 ioVGA_vsync	: out std_logic; -- vertical synchronisation
			 ioVGA_rgb		: out std_logic_vector (2 downto 0)
			 );
end softcore_top;



architecture behave_softcore_top of softcore_top is

-- internal reset signal: BTN 3 is used for the reset
signal reset			: std_logic;
-- output signal for input clock buffer
signal clk_bufg		: std_logic;

-- number of available interrupts in the system
-- limited by the data width of the wishbone system (32 bit)
-- so max 32 interrupts are possible
constant irqLines		: integer := 5;

-- wishbone implementation for the core
signal master_out		: wb_master_out;
signal master_in		: wb_master_in;

-- using a dummy to set signals to zero if not used by a wishbone device
signal dummy_out		: wb_master_in;

-- wishbone implementation for the interrupt controller
signal irq_out			: wb_master_in;
signal irq_in			: wb_master_out;
-- input bus for the interrupt lines of other wishbone devices
signal irq				: std_logic_vector (irqLines - 1 downto 0);
-- interrupt line to the core
signal irq_core		: std_logic;

-- wishbone implementation for the UART interface
signal uart1_out		: wb_master_in;
signal uart1_in		: wb_master_out;
signal irq_uart1		: std_logic;

-- wishbone implementation for the PS/2 interface
signal ps2_out			: wb_master_in;
signal ps2_in			: wb_master_out;
signal irq_ps2			: std_logic;

-- wishbone implementation for the onboard IOs
signal intIO_out		: wb_master_in;
signal intIO_in		: wb_master_out;
signal irq_intIO		: std_logic;

-- wishbone implementation for the SRAM1 interface
signal sram_out		: wb_master_in;
signal sram_in			: wb_master_out;
signal irq_sram		: std_logic;

-- wishbone implementation for the VGA interface
signal vga_out			: wb_master_in;
signal vga_in			: wb_master_out;
signal irq_vga			: std_logic;

for core : wb_core use entity work.wb_core (behave_zpu);


begin

	-- internal reset signal: BTN 3 is used for the reset
	reset <= ioButtons (3);

	-- Dedicated Input Clock Buffer
	-- (Using Global Clock Buffers: Instantiating Buffers Driven from Internal Logic
	-- Some synthesis tools require you to instantiate a global buffer in your code to use the
	-- dedicated routing resource if a high-fanout signal is sourced from internal flip-flops
	-- or logic (such as a clock divider or multiplexed clock), or if a clock is driven from a
	-- non-dedicated I/O pin.)
	BUFG_1: BUFG
		port map (
			I => clk,
			O => clk_bufg
		);

	-- wishbone master: the ZPU core
	core: wb_core
		port map (
			clk 			=> clk_bufg,
			reset 		=> reset,
			master_out 	=> master_out,
			master_in  	=> master_in,
			interrupt 	=> irq_core
		);

	-- the interrupt controller
	interrupt_inst: interrupt
		generic map (irqLines => irqLines)
		port map (
			clk 			=> clk_bufg,
			reset 		=> reset,
			irq_core 	=> irq_core,
			irq 			=> irq,
			slave_in 	=> irq_in,
			slave_out	=> irq_out
		);
	
	-- all lines are always connected to the wishbone but the cyc-signal is only
	-- set when the addr contains the address of the device
	irq_in.addr <= master_out.addr;
	irq_in.data <= master_out.data;
	irq_in.sel  <= master_out.sel;
	irq_in.we   <= master_out.we;
	irq_in.stb  <= master_out.stb;
	-- device address: 1000 0000 0000 0000 0000 0XXX XXXX XXXX
	--					   0x8	 0    0    0    0    -    X    X
	-- the leading 1000 is fix. The following bit is 1 when the core is reading or writing
	irq_in.cyc  <= master_out.cyc when (master_out.addr (31 downto 11) = "1000" & '0' & std_logic_vector (to_unsigned (0, (26 - 11 + 1)))) else '0';
	
	-- connecting the interrupt lines of the devices to the interrupt controllers bus
	irq(4) <= irq_vga;
	irq(3) <= irq_sram;
	irq(1) <= irq_uart1;
	irq(2) <= irq_ps2;
	irq(0) <= irq_intIO;


	-- the UART interface
	uart_inst_1: uart
		generic map (brdivisor => brdivisor1)
		port map (
			clk 			=> clk_bufg,
			reset 		=> reset,
			interrupt 	=> irq_uart1,
			uart_rx 		=> uart_rx1,
			uart_tx 		=> uart_tx1,
			slave_in 	=> uart1_in,
			slave_out 	=> uart1_out
		);

	-- all lines are always connected to the wishbone but the cyc-signal is only
	-- set when the addr contains the address of the device
	uart1_in.addr <= master_out.addr;
	uart1_in.data <= master_out.data;
	uart1_in.sel  <= master_out.sel;
	uart1_in.we   <= master_out.we;
	uart1_in.stb  <= master_out.stb;
	-- device address: 1000 0000 0001 0000 0001 0XXX XXXX XXXX
	--					   0x8	 0    1    0    1    -    X    X
	-- the leading 1000 is fix. The following bit is 1 when the core is reading or writing
	uart1_in.cyc  <= master_out.cyc when (master_out.addr (31 downto 11) = "1000" & '0' & "0000001000000010") else '0';
	
	
	
	ps2_inst: ps2
		port map (
			clk	 		=> clk_bufg,
			reset 		=> reset,
			interrupt 	=> irq_ps2,
			slave_in 	=> ps2_in,
			slave_out 	=> ps2_out,
			ps2clk		=> ps2_clk,
			DataIn 		=> ps2_data
		);

	-- all lines are always connected to the wishbone but the cyc-signal is only
	-- set when the addr contains the address of the device
	ps2_in.addr <= master_out.addr;
	ps2_in.data <= master_out.data;
	ps2_in.sel  <= master_out.sel;
	ps2_in.we   <= master_out.we;
	ps2_in.stb  <= master_out.stb;
	-- device address: 1000 0000 0001 0000 0000 0XXX XXXX XXXX
	--					   0x8	 0    1    0    0    -    X    X
	-- the leading 1000 is fix. The following bit is 1 when the core is reading or writing
	ps2_in.cyc  <= master_out.cyc when (master_out.addr (31 downto 11) = "1000" & '0' & "0000001000000000") else '0';
	
	
	intIO_inst: intIO
		port map (
			clk 			=> clk_bufg,
			reset 		=> reset,
			interrupt 	=> irq_intIO,
			slave_in 	=> intIO_in,
			slave_out 	=> intIO_out,
			LEDs 			=> ioLEDs,
			switches 	=> ioSwitches,
			sseg_on 		=> sseg_on,
			sseg 			=> sseg,
			buttons 		=> ioButtons
		);
		
	-- all lines are always connected to the wishbone but the cyc-signal is only
	-- set when the addr contains the address of the device
	intIO_in.addr <= master_out.addr;
	intIO_in.data <= master_out.data;
	intIO_in.sel  <= master_out.sel;
	intIO_in.we   <= master_out.we;
	intIO_in.stb  <= master_out.stb;
	-- device address: 1000 0000 0000 1000 0000 0XXX XXXX XXXX
	--					   0x8	 0    0    8    0    -    X    X
	-- the leading 1000 is fix. The following bit is 1 when the core is reading or writing
	intIO_in.cyc  <= master_out.cyc when (master_out.addr (31 downto 11) = "1000" & '0' & "0000000100000000") else '0';

	
	
	sram_inst: wb_sram 
		port map (
			clk 			=> clk_bufg,
			reset 		=> reset,
			interrupt 	=> irq_sram,
			slave_in 	=> sram_in,
			slave_out 	=> sram_out,
			ad 			=> sram_adr,
			we_n 			=> sram_we_n,
			oe_n 			=> sram_oe_n,
			dio_a 		=> sram_dio_a,
			ce_a_n 		=> sram_ce_a_n,
			ub_a_n 		=> sram_ub_a_n,
			lb_a_n 		=> sram_lb_a_n
		);
	
	-- all lines are always connected to the wishbone but the cyc-signal is only
	-- set when the addr contains the address of the device
	sram_in.addr <= master_out.addr;
	sram_in.data <= master_out.data;
	sram_in.sel  <= master_out.sel;
	sram_in.we   <= master_out.we;
	sram_in.stb  <= master_out.stb;
	-- device address: 1000 0000 0000 1100 0000 0XXX XXXX XXXX
	--					   0x8	 0    0    C    0    -    X    X
	-- the leading 1000 is fix. The following bit is 1 when the core is reading or writing
	sram_in.cyc  <= master_out.cyc when (master_out.addr (31 downto 11) = "1000" & '0' & "0000000110000000") else '0';
	
	
	vga_inst: wb_vga
		port map (
			clk 			=> clk_bufg,
			reset 		=> reset,
			interrupt 	=> irq_vga,
			slave_in 	=> vga_in,
			slave_out 	=> vga_out,
			hsync 		=> ioVGA_hsync,
			vsync 		=> ioVGA_vsync,
			rgb 			=> ioVGA_rgb
		);
	
	-- all lines are always connected to the wishbone but the cyc-signal is only
	-- set when the addr contains the address of the device
	vga_in.addr <= master_out.addr;
	vga_in.data <= master_out.data;
	vga_in.sel  <= master_out.sel;
	vga_in.we   <= master_out.we;
	vga_in.stb  <= master_out.stb;
	-- device address: 1000 0000 0000 1110 0000 0XXX XXXX XXXX
	--					   0x8	 0    0    E    0    -    X    X
	-- the leading 1000 is fix. The following bit is 1 when the core is reading or writing
	vga_in.cyc  <= master_out.cyc when (master_out.addr (31 downto 11) = "1000" & '0' & "0000000111000000") else '0';
	
	

	-- the dummy is connected to the master if there is no device matching to the current address
	dummy_out.ack <= '0';
	dummy_out.data <= (others => '0');

	-- connecting output signals of the devices to the master if they are selected by their address
	master_in <=
		irq_out		when (master_out.addr (31 downto 11) = "1000" & '0' & std_logic_vector (to_unsigned (0, (26 - 11 + 1)))) else
		uart1_out	when (master_out.addr (31 downto 11) = "1000" & '0' & "0000001000000010") else
		ps2_out		when (master_out.addr (31 downto 11) = "1000" & '0' & "0000001000000000") else
		intIO_out	when (master_out.addr (31 downto 11) = "1000" & '0' & "0000000100000000") else
		sram_out		when (master_out.addr (31 downto 11) = "1000" & '0' & "0000000110000000") else
		vga_out		when (master_out.addr (31 downto 11) = "1000" & '0' & "0000000111000000") else
		dummy_out;
		
end behave_softcore_top;
